
import argparse
import os
import json
import xml.etree.ElementTree as ET
def voc2coco(xml_folder, output_file):
    classes = {}
    images = []
    annotations = []
    sumatorio = 0
    for idx, filename in enumerate(os.listdir(xml_folder)):
        if not filename.endswith('.xml'):
continue
        xml_file = os.path.join(xml_folder, filename)
        tree = ET.parse(xml_file)
        root = tree.getroot()
        # Obtener información de la imagen
        image_id = os.path.splitext(filename)[0]
        width = int(root.find('size/width').text)
        height = int(root.find('size/height').text)
        images.append({
            'id': image_id,
            'file_name': f"{image_id}.jpg",
            'width': width,
            'height': height
        })
        sumatorio = int(sumatorio) + 1
        # Obtener información de las anotaciones
        for obj in root.findall('object'):
            cls_name = obj.find('name').text
            if cls_name not in classes:
                classes[cls_name] = len(classes) + 1
            cls_id = classes[cls_name]
            bbox = obj.find('bndbox')
            x1, y1 = float(bbox.find('xmin').text), float(bbox.find('ymin').text)
            x2, y2 = float(bbox.find('xmax').text), float(bbox.find('ymax').text)
            w, h = x2 - x1, y2 - y1
            annotations.append({
                'id': len(annotations) + 1,
                'image_id': image_id,
                'category_id': cls_id,
                'bbox': [x1, y1, w, h],
                'area': w * h,
                'iscrowd': 0
            })
    print("Se han procesado: ", sumatorio)
    # Crear el archivo COCO
    data = {
        'images': images,
        'annotations': annotations,
        'categories': [{'id': v, 'name': k} for k, v in classes.items()]
    }
    with open(output_file, 'w') as f:
        json.dump(data, f,indent=2)
voc2coco(".", “test.json") # Cambiar nombre según necesidad