# -*- coding: utf-8 -*-
"""Codigo_Detectron2.ipynb

Automatically generated by Colaboratory.

Original file is located at
    https://colab.research.google.com/drive/1TtFl_ToJdLABkNtG8i5mSAw6VCwd8nHl

# **INSTALAR DETECTRON2**
"""

!python -m pip install pyyaml==5.1
import sys, os, distutils.core

os.environ["TORCH_USE_CUDA_DSA"] = "1"

!git clone 'https://github.com/facebookresearch/detectron2'
dist = distutils.core.run_setup("./detectron2/setup.py")
!python -m pip install {' '.join([f"'{x}'" for x in dist.install_requires])}
sys.path.insert(0, os.path.abspath('./detectron2'))

import torch, detectron2
!nvcc --version
TORCH_VERSION = ".".join(torch.__version__.split(".")[:2])
CUDA_VERSION = torch.__version__.split("+")[-1]
print("torch: ", TORCH_VERSION, "; cuda: ", CUDA_VERSION)
print("detectron2:", detectron2.__version__)

# Some basico
# Setup detectron2 logger
import detectron2
from detectron2.utils.logger import setup_logger
setup_logger()

# importar librerías comunes
import numpy as np
import os, json, cv2, random
from google.colab.patches import cv2_imshow

# importar utilidades comunes de detectron2
from detectron2 import model_zoo
from detectron2.engine import DefaultPredictor
from detectron2.config import get_cfg
from detectron2.utils.visualizer import Visualizer
from detectron2.data import MetadataCatalog, DatasetCatalog

"""# **CARGAR NUEVO DATASET**"""

from detectron2.data.datasets import register_coco_instances
register_coco_instances("my_dataset_train", {}, "/content/drive/MyDrive/Dataset_2021_2022/train/train.json", "/content/drive/MyDrive/Dataset_2021_2022/train")
register_coco_instances("my_dataset_val", {}, "/content/drive/MyDrive/Dataset_2021_2022/validation/validation.json", "/content/drive/MyDrive/Dataset_2021_2022/validation")
register_coco_instances("my_dataset_test", {}, "/content/drive/MyDrive/Dataset_2021_2022/test/test.json", "/content/drive/MyDrive/Dataset_2021_2022/test")

"""# **CREACION DATASET, DATA AUGMENTATION Y COMPROBACIÓN ALEATORIA DE ANOTACIONES**"""

from detectron2.data import DatasetCatalog, MetadataCatalog

metadata_weed = MetadataCatalog.get("my_dataset_train")
dataset_dicts = DatasetCatalog.get("my_dataset_train")

metadata_val = MetadataCatalog.get("my_dataset_val")
dataset_dicts_val = DatasetCatalog.get("my_dataset_val")

from detectron2.data import transforms as T
from detectron2.data import MetadataCatalog

# Define la configuración de aumentos de datos
augmentation = [
    T.RandomBrightness(0.5, 2.0),
    T.RandomContrast(0.5, 2.0),
    T.RandomSaturation(0.5, 2.0),
    T.RandomLighting(0.0),
    T.RandomFlip(prob=0.5),
    T.RandomRotation(angle=[-30, 30]),
    T.RandomCrop("relative_range", (0.9, 0.9)),
]

# Aplica las transformaciones de aumentos de datos a las imágenes
for dataset_name in ["my_dataset_train", "my_dataset_val"]:
    MetadataCatalog.get(dataset_name).transforms = augmentation + [T.Resize((800, 800))]

"""
# **ENTRENADO DEL MODELO**"""

from detectron2.engine import DefaultTrainer
from detectron2.evaluation import COCOEvaluator, inference_on_dataset
from detectron2.data import build_detection_test_loader


cfg = get_cfg()
#seleccionar solo alguna de las dos líneas
#cfg.merge_from_file(model_zoo.get_config_file("COCO-Detection/faster_rcnn_R_50_FPN_3x.yaml"))
cfg.merge_from_file(model_zoo.get_config_file("COCO-Detection/retinanet_R_50_FPN_3x.yaml"))

cfg.DATASETS.TRAIN = ("my_dataset_train",)
cfg.DATASETS.TEST = ("my_dataset_val",)
cfg.DATALOADER.NUM_WORKERS = 2
#seleccionar solo alguna de las dos líneas
#cfg.MODEL.WEIGHTS = model_zoo.get_checkpoint_url("COCO-Detection/faster_rcnn_R_50_FPN_3x.yaml")  # Let training initialize from model zoo
cfg.MODEL.WEIGHTS = model_zoo.get_checkpoint_url("COCO-Detection/retinanet_R_50_FPN_3x.yaml")  # Let training initialize from model zoo


cfg.SOLVER.IMS_PER_BATCH = 6
cfg.SOLVER.BASE_LR = 0.0025  # Learning Rate
cfg.SOLVER.WARMUP_ITERS = 1000
cfg.SOLVER.MAX_ITER = 5000
cfg.SOLVER.STEPS = [2500,4000]
cfg.SOLVER.MOMENTUM = 0.9
cfg.SOLVER.GAMMA = 0.5
cfg.SOLVER.WEIGHT_DECAY = 0.0001  # Ajuste pesos



cfg.MODEL.ROI_HEADS.BATCH_SIZE_PER_IMAGE = 512
cfg.MODEL.RETINANET.BATCH_SIZE_PER_IMAGE = 512
cfg.MODEL.ROI_HEADS.NUM_CLASSES = 7
cfg.MODEL.RETINANET.NUM_CLASSES = 7


os.makedirs(cfg.OUTPUT_DIR, exist_ok=True)
trainer = DefaultTrainer(cfg)
trainer.resume_or_load(resume=False)
trainer.train()

# Commented out IPython magic to ensure Python compatibility.
# Look at training curves in tensorboard:
# %load_ext tensorboard
# %tensorboard --logdir output

"""# **PROBAMOS EL ENTRENAMIENTO**"""

from detectron2.engine import DefaultPredictor
from detectron2.utils.visualizer import Visualizer, ColorMode
import cv2
import random

# Carga el modelo entrenado
cfg.MODEL.WEIGHTS = os.path.join(cfg.OUTPUT_DIR, "model_final.pth")
cfg.MODEL.ROI_HEADS.SCORE_THRESH_TEST = 0.5   # set a custom testing threshold
cfg.MODEL.RETINANET.SCORE_THRESH_TEST = 0.5
predictor = DefaultPredictor(cfg)

# Carga algunas imágenes de validación para probar
from detectron2.data import DatasetCatalog, MetadataCatalog
dataset_dicts_test = DatasetCatalog.get("my_dataset_test")
metadata_test = MetadataCatalog.get("my_dataset_test")

for d in random.sample(dataset_dicts_test, 3):
    im = cv2.imread(d["file_name"])
    print(d["file_name"])

    # Realiza la predicción en la imagen
    outputs = predictor(im)

    # Visualiza las predicciones
    v = Visualizer(im[:, :, ::-1],
                   metadata=metadata_test,
                   scale=0.25,
                   instance_mode=ColorMode.IMAGE_BW)
    out = v.draw_instance_predictions(outputs["instances"].to("cpu"))
    cv2_imshow(out.get_image()[:, :, ::-1])

"""# **MÉTRICAS**"""

from detectron2.evaluation import COCOEvaluator, inference_on_dataset
from detectron2.data import build_detection_test_loader
evaluator = COCOEvaluator("my_dataset_test",cfg, False, output_dir="./output")
val_loader = build_detection_test_loader(cfg, "my_dataset_test")
print(inference_on_dataset(predictor.model, val_loader, evaluator))