# -*- coding: utf-8 -*-
"""Codigo_MMDetection.ipynb

Automatically generated by Colaboratory.

Original file is located at
    https://colab.research.google.com/drive/1FzY_CJsa8Zs5x_fzwAzrPe-xD2F7f6fX
"""

# Comprobar versión nvcc
!nvcc -V
# Comprobar versión GCC
!gcc --version

# Commented out IPython magic to ensure Python compatibility.
# instalar dependencias
!pip install torch==1.9.0+cu111 torchvision==0.10.0+cu111 -f https://download.pytorch.org/whl/torch_stable.html

# instalar mmcv-full para que podamos usar operadores CUDA
!pip install mmcv-full -f https://download.openmmlab.com/mmcv/dist/cu111/torch1.9.0/index.html --no-cache-dir

# Instalar mmdetection
!rm -rf mmdetection
!git clone -b 2.x --single-branch https://github.com/open-mmlab/mmdetection.git
# %cd mmdetection

!pip install -e .

# recoger variables de entorno
from mmcv import collect_env
collect_env()

# Comprobar instalación de Pytorch
import torch, torchvision
print(torch.__version__, torch.cuda.is_available())

# Comprobar instalación de MMDetection
import mmdet
print(mmdet.__version__)

# Comprobar instalación de mmcv
from mmcv.ops import get_compiling_cuda_version, get_compiler_version
print(get_compiling_cuda_version())
print(get_compiler_version())

# Descarga de los checkpoints pre-entrendaos para inferir y finetuning
!mkdir checkpoints
!wget -c https://download.openmmlab.com/mmdetection/v2.0/faster_rcnn/faster_rcnn_r50_caffe_fpn_mstrain_3x_coco/faster_rcnn_r50_caffe_fpn_mstrain_3x_coco_20210526_095054-1f77628b.pth \
      -O checkpoints/faster_rcnn_r50_caffe_fpn_mstrain_3x_coco_20210526_095054-1f77628b.pth

!wget -c https://download.openmmlab.com/mmdetection/v2.0/retinanet/retinanet_r50_fpn_1x_coco/retinanet_r50_fpn_1x_coco_20200130-c2398f9e.pth -O checkpoints/retinanet_r50_fpn_1x_coco_20200130-c2398f9e.pth

!wget -c https://download.openmmlab.com/mmdetection/v2.0/yolo/yolov3_d53_mstrain-608_273e_coco/yolov3_d53_mstrain-608_273e_coco-139f5633.pth -O checkpoints/yolov3_d53_mstrain-608_273e_coco-139f5633.pth

"""# **CONFIGURACIÓN PARA RETINANET**"""

#Ejecutar solo una de las configuraciones

from mmcv import Config
cfg = Config.fromfile('./configs/retinanet/retinanet_r50_fpn_1x_coco.py')

from mmdet.apis import set_random_seed

# Modificar el tipo de dataset y las rutas
cfg.dataset_type = 'CocoDataset'
cfg.data_root = '/content/drive/MyDrive/Dataset_2021_2022/'

cfg.data.test.type = 'CocoDataset'
cfg.data.test.data_root = '/content/drive/MyDrive/Dataset_2021_2022/'
cfg.data.test.ann_file = 'test/test.json'
cfg.data.test.img_prefix = 'test/'

cfg.data.train.type = 'CocoDataset'
cfg.data.train.data_root = '/content/drive/MyDrive/Dataset_2021_2022/'
cfg.data.train.ann_file = 'train/train.json'
cfg.data.train.img_prefix = 'train/'

cfg.data.val.type = 'CocoDataset'
cfg.data.val.data_root = '/content/drive/MyDrive/Dataset_2021_2022/'
cfg.data.val.ann_file = 'validation/validation.json'
cfg.data.val.img_prefix = 'validation/'

# Cargamos el modelo pre-entrenado
cfg.load_from = 'checkpoints/retinanet_r50_fpn_1x_coco_20200130-c2398f9e.pth'
# Ruta para guardar ficheros y logs.
cfg.work_dir = './tutorial_exps'

# Valor del learning rate
cfg.optimizer.lr = 0.00025 #retina

cfg.model.bbox_head.num_classes = 7  # Número total de clases en tu conjunto de datos (retina y yolo)
cfg.data.train.classes = ['SETIT', 'SOLNI', 'POROL', 'LYPES', 'NCI', 'CYP_CYPRO', 'ECHCG']
cfg.data.val.classes = ['SETIT', 'SOLNI', 'POROL', 'LYPES', 'NCI', 'CYP_CYPRO', 'ECHCG']
cfg.data.test.classes = ['SETIT', 'SOLNI', 'POROL', 'LYPES', 'NCI', 'CYP_CYPRO', 'ECHCG']

# Modificación
cfg.data.train.pipeline = [
    dict(type='LoadImageFromFile'),
    dict(type='LoadAnnotations', with_bbox=True),
    dict(type='Resize', img_scale=(1333, 1000), keep_ratio=True),
    dict(type='RandomFlip', flip_ratio=0.5),
    dict(type='Normalize', mean= [123.675, 116.28, 103.53], std= [58.395, 57.12, 57.375], to_rgb= True),
    dict(type='Pad', size_divisor=32),
    dict(type='DefaultFormatBundle'),
    dict(type='Collect', keys=['img', 'gt_bboxes', 'gt_labels']),
]

cfg.optimizer.weight_decay = 0.001  # Prueba con diferentes valores

cfg.lr_config.warmup = None
cfg.log_config.interval = 50

#Forzamos que la backbone para sacar caracteristicas sea ResNet con
#una profundidad de 50
cfg.model.backbone.type = 'ResNet'
cfg.model.backbone.depth = 50


# Usamos la métrica bbox (que incluye AP, AP50 y AP75) y con classwise
#indicamos que nos muestre las métricas para cada clase
cfg.evaluation = dict(
    interval=1,
    metric='bbox',
    classwise=True,
    iou_thrs=[0.5]
)


# Indicamos que muestre las métricas cada 1 epoch y que el entrenamiento dure 20 epochs
cfg.checkpoint_config.interval = 1
cfg.runner.max_epochs = 40
cfg.total_epochs = 40

# Establecer el seed para que los resultados sean mas repoducibles
cfg.seed = 0
set_random_seed(0, deterministic=False)
cfg.gpu_ids = range(1)

# Código necesario para mostrar los resultados en tensorboard
cfg.log_config.hooks = [
    dict(type='TextLoggerHook'),
    dict(type='TensorboardLoggerHook')]


# Mostramos el log del entrenamiento
print(f'Config:\n{cfg.pretty_text}')

"""# **CONFIGURACIÓN PARA FASTER R-CNN**"""

#Ejecutar solo una de las configuraciones

from mmcv import Config
cfg = Config.fromfile('./configs/faster_rcnn/faster_rcnn_r50_caffe_fpn_mstrain_1x_coco.py')

from mmdet.apis import set_random_seed

# Modificar el tipo de dataset y las rutas
cfg.dataset_type = 'CocoDataset'
cfg.data_root = '/content/drive/MyDrive/Dataset_2021_2022/'

cfg.data.test.type = 'CocoDataset'
cfg.data.test.data_root = '/content/drive/MyDrive/Dataset_2021_2022/'
cfg.data.test.ann_file = 'test/test.json'
cfg.data.test.img_prefix = 'test/'

cfg.data.train.type = 'CocoDataset'
cfg.data.train.data_root = '/content/drive/MyDrive/Dataset_2021_2022/'
cfg.data.train.ann_file = 'train/train.json'
cfg.data.train.img_prefix = 'train/'

cfg.data.val.type = 'CocoDataset'
cfg.data.val.data_root = '/content/drive/MyDrive/Dataset_2021_2022/'
cfg.data.val.ann_file = 'validation/validation.json'
cfg.data.val.img_prefix = 'validation/'

# Se carga el modelo pre-entrenado
cfg.load_from = 'checkpoints/faster_rcnn_r50_caffe_fpn_mstrain_3x_coco_20210526_095054-1f77628b.pth'

# Ruta para guardar ficheros y logs.
cfg.work_dir = './tutorial_exps'

# Valor del learning rate
cfg.optimizer.lr = 0.00025


# Modifícanos número de clases y clases
cfg.model.roi_head.bbox_head.num_classes = 7
cfg.data.train.classes = ['SETIT', 'SOLNI', 'POROL', 'LYPES', 'NCI', 'CYP_CYPRO', 'ECHCG']
cfg.data.val.classes = ['SETIT', 'SOLNI', 'POROL', 'LYPES', 'NCI', 'CYP_CYPRO', 'ECHCG']
cfg.data.test.classes = ['SETIT', 'SOLNI', 'POROL', 'LYPES', 'NCI', 'CYP_CYPRO', 'ECHCG']

cfg.data.train.pipeline = [
    dict(type='LoadImageFromFile'),
    dict(type='LoadAnnotations', with_bbox=True),
    dict(type='Resize', img_scale=(1333, 1000), keep_ratio=True),
    dict(type='RandomFlip', flip_ratio=0.5),
    dict(type='Normalize', mean= [123.675, 116.28, 103.53], std= [58.395, 57.12, 57.375], to_rgb= True),
    dict(type='Pad', size_divisor=32),
    dict(type='DefaultFormatBundle'),
    dict(type='Collect', keys=['img', 'gt_bboxes', 'gt_labels']),
]

cfg.lr_config.warmup = None
cfg.log_config.interval = 100

cfg.evaluation = dict(
    interval=1,
    metric='bbox',
    classwise=True,  # Agrega esta línea para habilitar classwise mAP
    iou_thrs=[0.5]
)

# Intervalo para guardar el checkpoint y epochs del entrenamiento
cfg.checkpoint_config.interval = 1
cfg.runner.max_epochs = 20
cfg.total_epochs = 20

# Set del seed para que los resultados sean mas repoducibles
cfg.seed = 0
set_random_seed(0, deterministic=False)
cfg.gpu_ids = range(1)

# Guardar los logs para usar en tensorboard
cfg.log_config.hooks = [
    dict(type='TextLoggerHook'),
    dict(type='TensorboardLoggerHook')]


#Mostramos resultados
print(f'Config:\n{cfg.pretty_text}')

"""# **CONFIGURACIÓN PARA YOLOV3**"""

#Ejecutar solo una de las configuraciones

from mmcv import Config
cfg = Config.fromfile('./configs/yolo/yolov3_d53_mstrain-608_273e_coco.py')

from mmdet.apis import set_random_seed

# Modificar el tipo de dataset y las rutas
cfg.dataset_type = 'CocoDataset'
cfg.data_root = '/content/drive/MyDrive/Dataset_2021_2022/'

cfg.data.test.type = 'CocoDataset'
cfg.data.test.data_root = '/content/drive/MyDrive/Dataset_2021_2022/'
cfg.data.test.ann_file = 'test/test.json'
cfg.data.test.img_prefix = 'test/'

cfg.data.train.type = 'CocoDataset'
cfg.data.train.data_root = '/content/drive/MyDrive/Dataset_2021_2022/'
cfg.data.train.ann_file = 'train/train.json'
cfg.data.train.img_prefix = 'train/'

cfg.data.val.type = 'CocoDataset'
cfg.data.val.data_root = '/content/drive/MyDrive/Dataset_2021_2022/'
cfg.data.val.ann_file = 'validation/validation.json'
cfg.data.val.img_prefix = 'validation/'


# Se carga el modelo pre-entrenado
cfg.load_from = 'checkpoints/yolov3_d53_mstrain-608_273e_coco-139f5633.pth'
# Ruta para guardar ficheros y log
cfg.work_dir = './tutorial_exps'

# Valor del learning rate
cfg.optimizer.lr = 0.00025  #yolo

cfg.optimizer.weight_decay = 0.001  # Prueba con diferentes valores

cfg.model.bbox_head.num_classes = 7 #retina y yolo
cfg.data.train.classes = ['SETIT', 'SOLNI', 'POROL', 'LYPES', 'NCI', 'CYP_CYPRO', 'ECHCG']
cfg.data.val.classes = ['SETIT', 'SOLNI', 'POROL', 'LYPES', 'NCI', 'CYP_CYPRO', 'ECHCG']
cfg.data.test.classes = ['SETIT', 'SOLNI', 'POROL', 'LYPES', 'NCI', 'CYP_CYPRO', 'ECHCG']


cfg.lr_config.warmup = None
cfg.log_config.interval = 50

cfg.evaluation = dict(
    interval=1,
    metric='bbox',
    classwise=True,  # Agrega esta línea para habilitar classwise mAP
    iou_thrs=[0.5]
)

# Intervalo para guardar el checkpoint y epochs del entrenamiento
cfg.checkpoint_config.interval = 1
cfg.runner.max_epochs = 40
cfg.total_epochs = 40

# Set del seed para que los resultados sean mas repoducibles
cfg.seed = 0
set_random_seed(0, deterministic=False)
cfg.gpu_ids = range(1)

# Guardar los logs para usar en tensorboard
cfg.log_config.hooks = [
    dict(type='TextLoggerHook'),
    dict(type='TensorboardLoggerHook')]


#Mostramos resultados
print(f'Config:\n{cfg.pretty_text}')

"""# **ENTRENAMIENTO**"""

# Añadido por mi porque sino daba error el siguiente paso
cfg.device='cuda'

from mmdet.datasets import build_dataset
from mmdet.models import build_detector
from mmdet.apis import train_detector
import os.path as osp
import mmcv


# Construir dataset
datasets = [build_dataset(cfg.data.train)]

# Construir detector
model = build_detector(cfg.model)
model.CLASSES = datasets[0].CLASSES

# Crear directorio
mmcv.mkdir_or_exist(osp.abspath(cfg.work_dir))
train_detector(model, datasets, cfg, distributed=False, validate=True)

# Commented out IPython magic to ensure Python compatibility.
# Cargar tensorboard en Colab
# %load_ext tensorboard

# Visualizar
# %tensorboard --logdir ./tutorial_exps

from mmdet.apis import init_detector, inference_detector, show_result_pyplot
import mmcv
import os
import random
import time

#Inferencia con una imagen
img = mmcv.imread('/content/drive/MyDrive/Dataset_2021_2022/test/IMG_5707.JPG')
start_time = time.time()
model.cfg = cfg
result = inference_detector(model, img)
end_time = time.time()

inference_time = end_time - start_time
print(f"Tiempo de inferencia: {inference_time} segundos")

#Mostrar resultado
show_result_pyplot(model, img, result,score_thr=0.5)
model.show_result(img, result, out_file='prueba.jpg')